local dragon = require('dragon')
local fly = require('fly')

function generateSeedFromClock()
	local seed = os.time() + math.floor(1000 * os.clock())
	seed = seed * seed % 1000000
	seed = seed * seed % 1000000
	return seed
end

function love.load()
	math.randomseed(generateSeedFromClock())
	w, h = love.graphics.getDimensions()
	scoreFont = love.graphics.newFont('RobotoSlab-Regular.ttf', 48)
	unitFont = love.graphics.newFont('RobotoSlab-Regular.ttf', 12)
	time, catchTimes = 0, { }
	score = false
	flies = {}
	mama = dragon.new(w/2, h/2, 30)
end

function love.keypressed(key)
	local alt = love.keyboard.isDown('lalt') or love.keyboard.isDown('ralt')
	if key == 'escape' then love.event.quit()
	elseif key == 'f12' or (alt and key == 'return') then
		local oldw, oldh = w, h
		love.window.setFullscreen(not love.window.getFullscreen())
		w, h = love.graphics.getDimensions()
		mama:onresize(oldw, oldh)
		for i=1,#flies do flies[i]:onresize(oldw, oldh) end
	end
end

function toNum(flag) return flag and 1 or 0 end

function arrows()
	local  left = toNum(love.keyboard.isDown('left'))
	local right = toNum(love.keyboard.isDown('right'))
	local    up = toNum(love.keyboard.isDown('up'))
	local  down = toNum(love.keyboard.isDown('down'))
	return right - left, down - up
end

-- Find the time (if any) when two moving circles will collide.
function circlesCollide(c0, c1)
	-- offset and relative velocity
	local ox, oy = c1.x - c0.x, c1.y - c0.y
	local vx, vy = c1.vx - c0.vx, c1.vy - c0.vy
	local r = c0.r + c1.r

	-- Find the time when |o + t*v| = r, i.e. square both sides
	-- and solve the resulting quadratic:
	-- t^2*|v|^2 + 2*t*dot(v,o) + |p|^2 - r^2 = 0.
	local a = vx*vx + vy*vy
	local b = 2*(vx*ox + vy*oy)
	local c = (ox*ox + oy*oy) - r*r
	if c < 0 then return 0 end  -- They are already overlapping.
	local t = solveQuadratic(a, b, c)
	if t then
		if t[1] >= 0 and t[1] < t[2] then return t[1]
		elseif t[2] >= 0 then return t[2] end
	end
	return false
end

-- If there are real-valued solutions, return them in a 2-element array.
function solveQuadratic(a, b, c)
	-- Divide by a, then halve b to get: x^2 - 2bx + c.
	-- Then solutions are: b +/- sqrt(b^2 - c).
	b, c = -0.5*b/a, c/a
	-- If the discriminant is negative there are no real solutions.
	local d = b*b - c;  if d < 0 then return end
	-- For numerical stability, use the one with larger magnitude.
	local x0 = b + (b < 0 and -1 or 1) * math.sqrt(d)
	-- Notice that x0*x1 = b^2 - (b^2 - c) = c, so x1 = c/x0
	return { x0, c/x0 }
end

function collides(d, f, dt)
	local t = circlesCollide(d, f)
	return t and t < dt
end

function numFliesWanted()
	return math.floor(0.5 + math.sqrt(w*h)/100)
end

function love.update(dt)
	dt = math.min(0.1, dt)
	time = time + dt
	local n = numFliesWanted()
	for i=#flies+1,n do
		table.insert(flies, fly.new())
	end
	local ax, ay = arrows()
	mama:update(ax, ay, dt)
	for i=#flies,1,-1 do
		flies[i]:update(dt)
		if collides(mama, flies[i], dt) then
			table.insert(catchTimes, time)
			if #flies > n then table.remove(flies, i)
			else flies[i] = fly.new() end
		end
	end
	while #catchTimes > 0 and catchTimes[1] < time - 20 do
		table.remove(catchTimes, 1)
	end
	if #catchTimes == 0 then score = 0 else
		local minutes = math.max(5, time - catchTimes[1]) / 60
		score = math.floor(0.5 + #catchTimes / minutes )
	end
end

function love.draw()
	love.graphics.setColor(190, 200, 90)
	local s = tostring(score)
	if score then
		for i=string.len(s)-3,1,-3 do
			s = string.sub(s, 1, i) .. ',' .. string.sub(s, i+1)
		end
	else s = '???' end
	love.graphics.setFont(scoreFont)
	love.graphics.print(s, w-60 - scoreFont:getWidth(s), 60)
	love.graphics.setFont(unitFont)
	love.graphics.print('flies/min', w-55, 100)
	love.graphics.setColor(150, 150, 150)
	for i=1,#flies do flies[i]:draw() end
	love.graphics.setColor(200, 45, 30)
	mama:draw()
end
